Attribute VB_Name = "physics1"
Option Explicit
Public pts(10) As pointObj
Public sps(5) As spring
Public walls(4) As wall

Public Const maxPts = 3
Public Const maxSprings = 5
Public Const maxWalls = 3

Public opt_k As Single
Public opt_d As Single
Public opt_mass As Single
Public opt_friction As Single
Public opt_bounce As Single
Public opt_length As Single
Public opt_gravity As Single

Public Sub initVariables()
opt_k = 0.2
opt_d = 0.2
opt_mass = 1
opt_friction = 0.7
opt_bounce = 0.8
opt_length = 100
opt_gravity = 0.1
End Sub

Public Sub initSystem()
Static i As Integer
    For i = 0 To maxPts
        pts(i).mass = opt_mass
    Next i
    
    For i = 0 To maxSprings
        sps(i).d = opt_d
        sps(i).k = opt_k
        sps(i).mid_length = opt_length
    Next i
    sps(1).mid_length = Sqr(2) * opt_length
    sps(5).mid_length = Sqr(2) * opt_length
    
    For i = 0 To maxWalls
        walls(i).bounce = opt_bounce
        walls(i).friction = opt_friction
    Next i
    
    pts(0).pos.x = 1
    pts(0).pos.y = 1
    
    pts(1).pos.x = 1
    pts(1).pos.y = 1 + opt_length
    
    pts(2).pos.x = 1 + opt_length
    pts(2).pos.y = 1
   
    pts(3).pos.x = 1 + opt_length
    pts(3).pos.y = 1 + opt_length
    
    pts(4).pos.x = 1 + Sqr(2) * opt_length
    pts(4).pos.y = 1 + Sqr(2) * opt_length
      
    sps(0).node1 = 0
    sps(0).node2 = 1
   
    sps(1).node1 = 1
    sps(1).node2 = 2
    
    sps(2).node1 = 2
    sps(2).node2 = 0
    
    sps(3).node1 = 2
    sps(3).node2 = 3
    
    sps(4).node1 = 1
    sps(4).node2 = 3
    
    sps(5).node1 = 0
    sps(5).node2 = 3
    
    With walls(0)
        .pt1.x = 0
        .pt1.y = 0
        .pt2.x = 700
        .pt2.y = 0
    End With
    
    With walls(1)
        .pt1.x = 700
        .pt1.y = 0
        .pt2.x = 100
        .pt2.y = 500
    End With
    
    With walls(2)
        .pt1.x = 700
        .pt1.y = 500
        .pt2.x = 0
        .pt2.y = 500
    End With
    With walls(3)
        .pt1.x = 0
        .pt1.y = 500
        .pt2.x = 0
        .pt2.y = 0
    End With
End Sub

Public Sub drawSpring(sprg As spring, p As PictureBox)
With sprg
    p.Circle (pts(.node1).pos.x, pts(.node1).pos.y), 10
    p.Circle (pts(.node2).pos.x, pts(.node2).pos.y), 10
   
    p.Line (pts(.node1).pos.x, pts(.node1).pos.y)-(pts(.node2).pos.x, pts(.node2).pos.y)
End With
End Sub

Public Sub drawWall(w As wall, p As PictureBox)
    With w
        p.Line (.pt1.x, .pt1.y)-(.pt2.x, .pt2.y)
    End With
End Sub

Public Sub drawSystem(p As PictureBox)
Static i As Integer
    For i = 0 To maxSprings
        drawSpring sps(i), p
    Next i
    For i = 0 To maxWalls
        drawWall walls(i), p
    Next i
    
End Sub

Public Sub simulateSystem()
Dim i As Integer
    For i = 0 To maxPts
        pts(i).force.y = pts(i).force.y + opt_gravity * pts(i).mass
    Next i
    
    For i = 0 To maxSprings
        simulateSpring sps(i)
    Next i
    
    For i = 0 To maxPts
        applyForces pts(i)
    Next i
End Sub

Public Function collideWall2d(w As wall, p As pointObj) As Boolean
'warning: this function changes the velocity of p by reference
Dim cv1 As Vector3d, cv2 As Vector3d
Dim r1 As Vector3d, r2 As Vector3d
Dim s As Vector3d

s = vectorUnit(vectorSub(w.pt2, w.pt1))

r1 = vectorSub(p.pos, w.pt1) 'old position
r2 = vectorSub(vectorAdd(p.pos, p.vel), w.pt1) 'new position
cv1 = vectorUnit(vectorCross(r1, s))
cv2 = vectorUnit(vectorCross(r2, s))

If Not vectorIntEqual(cv1, cv2) Then 'angle between the vectors has flipped
    p.vel = vectorSub(vectorScale(vectorComponentParallel(p.vel, s), w.friction), _
        vectorScale(vectorComponentPerpendicular(p.vel, s), w.bounce))
End If

End Function

Public Sub simulateSpring(sprg As spring)
    Dim clen As Single, delta As Single
    Dim f As Single 'force
    Dim directionV As Vector3d
    Dim damping As Single
    
    'directionV is unitVector from node2 to node1.
    directionV = vectorUnit(vectorSub(pts(sprg.node1).pos, pts(sprg.node2).pos))
    'clen is current length of spring
    clen = vectorDistance(pts(sprg.node1).pos, pts(sprg.node2).pos)
    'delta is the diff between median length and current length
    'delta +ve is for impending expansion, -ve for impending compression.
    delta = sprg.mid_length - clen
    
    'v2-v1 is velocity of node 2 wrt node 1. if dot of this with directionV is +ve
    'then impending compression. else impending expansion
    'damping is the force damping depending on this velocity.
    'sprg.d is damping factor of spring
    damping = sprg.d * vectorDot(vectorSub(pts(sprg.node2).vel, pts(sprg.node1).vel), directionV)
    'f is the force due to compression/expansion of spring.
    f = sprg.k * delta

    pts(sprg.node1).force = vectorAdd(pts(sprg.node1).force, vectorScale(directionV, f + damping)) 'vectorSub(vectorScale(directionV, f - damping), pts(sprg.node1).vel))
    pts(sprg.node2).force = vectorAdd(pts(sprg.node2).force, vectorScale(directionV, -f - damping)) 'vectorSub(vectorScale(directionV, -f + damping), pts(sprg.node2).vel))
End Sub

Public Sub applyForces(p As pointObj)
Static i As Integer
    p.vel = vectorAdd(p.vel, vectorScale(p.force, 1 / p.mass))
    For i = 0 To maxWalls
        collideWall2d walls(i), p
    Next i
    p.pos = vectorAdd(p.pos, p.vel)
    p.force = createVector(0, 0, 0)
End Sub

